@@ -17,9 +17,9 @@ from libcst import (
     RemoveFromParent,
 )
 from libcst import matchers as m
-from libcst.codemod import ContextAwareTransformer
+from libcst.codemod import CodemodContext, ContextAwareTransformer
 from libcst.codemod.visitors import AddImportsVisitor
-from libcst.metadata import Scope, ScopeProvider
+from libcst.metadata import ParentNodeProvider, Scope, ScopeProvider
 
 
 class BaseDjCodemodTransformer(ContextAwareTransformer, ABC):
@@ -27,7 +27,9 @@ class BaseDjCodemodTransformer(ContextAwareTransformer, ABC):
     removed_in: Tuple[int, int]
 
 
-def module_matcher(import_parts: Sequence) -> Union[m.BaseMatcherNode, m.DoNotCare]:
+def module_matcher(
+    import_parts: Sequence[str],
+) -> Union[m.Attribute, m.Name]:
     """Build matcher for a module given sequence of import parts."""
     # If only one element, it is just a Name
     if len(import_parts) == 1:
@@ -37,7 +39,7 @@ def module_matcher(import_parts: Sequence) -> Union[m.BaseMatcherNode, m.DoNotCa
     return m.Attribute(value=value, attr=m.Name(attr))
 
 
-def import_from_matches(node: ImportFrom, module_parts: Sequence) -> bool:
+def import_from_matches(node: ImportFrom, module_parts: Sequence[str]) -> bool:
     """Check if an `ImportFrom` node matches sequence of module parts."""
     return m.matches(node, m.ImportFrom(module=module_matcher(module_parts)))
 
@@ -50,25 +52,12 @@ class BaseRenameTransformer(BaseDjCodemodTransformer, ABC):
 
     simple_rename = True
 
-    @property
-    def old_name(self):
-        return self.rename_from.split(".")[-1]
-
-    @property
-    def old_module_parts(self):
-        return self.rename_from.split(".")[:-1]
-
-    @property
-    def new_name(self):
-        return self.rename_to.split(".")[-1]
-
-    @property
-    def new_module_parts(self):
-        return self.rename_to.split(".")[:-1]
-
-    @property
-    def ctx_key_imported_as(self):
-        return f"{self.rename_from}-imported_as"
+    def __init__(self, context: CodemodContext) -> None:
+        super().__init__(context)
+        *self.old_module_parts, self.old_name = self.rename_from.split(".")
+        *self.new_module_parts, self.new_name = self.rename_to.split(".")
+        self.ctx_key_imported_as = f"{self.rename_from}-imported_as"
+        self.ctx_key_import_scope = f"{self.rename_from}-import_scope"
 
     @property
     def entity_imported_as(self):
@@ -83,8 +72,10 @@ class BaseRenameTransformer(BaseDjCodemodTransformer, ABC):
         self, original_node: ImportFrom, updated_node: ImportFrom
     ) -> Union[BaseSmallStatement, RemovalSentinel]:
         """Update import statements for matching old module name."""
-        if not import_from_matches(updated_node, self.old_module_parts):
-            return super().leave_ImportFrom(original_node, updated_node)
+        if not import_from_matches(updated_node, self.old_module_parts) or isinstance(
+            updated_node.names, ImportStar
+        ):
+            return updated_node
         # This is a match
         new_names = list(self.gen_new_imported_names(updated_node.names))
         self.save_import_scope(original_node)
@@ -96,8 +87,8 @@ class BaseRenameTransformer(BaseDjCodemodTransformer, ABC):
         return updated_node.with_changes(names=cleaned_names)
 
     def gen_new_imported_names(
-        self, old_names: Union[Sequence[ImportAlias], ImportStar]
-    ) -> Generator:
+        self, old_names: Sequence[ImportAlias]
+    ) -> Generator[ImportAlias, None, None]:
         """Update import if the entity we're interested in is imported."""
         for import_alias in old_names:
             if not self.old_name or import_alias.evaluated_name == self.old_name:
@@ -108,9 +99,13 @@ class BaseRenameTransformer(BaseDjCodemodTransformer, ABC):
                     continue
             yield import_alias
 
+    def resolve_parent_node(self, node: CSTNode) -> CSTNode:
+        parent_nodes = self.context.wrapper.resolve(ParentNodeProvider)  # type: ignore
+        return parent_nodes[node]
+
     def resolve_scope(self, node: CSTNode) -> Scope:
-        scopes_map = self.context.wrapper.resolve(ScopeProvider)
-        return scopes_map[node]
+        scopes_map = self.context.wrapper.resolve(ScopeProvider)  # type: ignore
+        return scopes_map[node]  # type: ignore
 
     def save_import_scope(self, import_from: ImportFrom) -> None:
         scope = self.resolve_scope(import_from)
@@ -120,10 +115,6 @@ class BaseRenameTransformer(BaseDjCodemodTransformer, ABC):
     def import_scope(self) -> Optional[Scope]:
         return self.context.scratch.get(self.ctx_key_import_scope, None)
 
-    @property
-    def ctx_key_import_scope(self) -> str:
-        return f"{self.rename_from}-import_scope"
-
     @staticmethod
     def tidy_new_imported_names(
         new_names: Sequence[ImportAlias],
@@ -148,64 +139,45 @@ class BaseRenameTransformer(BaseDjCodemodTransformer, ABC):
             asname=as_name,
         )
 
-    @property
-    def ctx_key_is_name_called(self):
-        return f"{self.rename_from}-is_name_called"
-
-    @property
-    def entity_is_name_called(self):
-        return self.context.scratch.get(self.ctx_key_is_name_called, False)
-
-    def visit_Call(self, node: Call) -> Optional[bool]:
-        if self.is_imported_with_old_name and m.matches(
-            node, m.Call(func=m.Name(self.old_name))
-        ):
-            self.context.scratch[self.ctx_key_is_name_called] = True
-        return None
-
-    def leave_Call(self, original_node: Call, updated_node: Call) -> BaseExpression:
-        self.context.scratch.pop(self.ctx_key_is_name_called, None)
-        return super().leave_Call(original_node, updated_node)
-
     def leave_Name(self, original_node: Name, updated_node: Name) -> BaseExpression:
+        """Rename reference to the imported name."""
         if (
             self.is_imported_with_old_name
-            and not self.entity_is_name_called
             and m.matches(updated_node, m.Name(value=self.old_name))
+            and not self.is_wrapped_in_call(original_node)
+            and self.matches_import_scope(original_node)
         ):
-            try:
-                scope = self.resolve_scope(original_node)
-            except KeyError:
-                # Can't resolve scope of original_node, ignore it
-                # Might be because of one of these reasons:
-                # - It's the same name in another scope
-                # - It's a attribute with the same name
-                # - It's a keyword argument
-                pass
-            else:
-                if scope == self.import_scope:
-                    return updated_node.with_changes(value=self.new_name)
+            return updated_node.with_changes(value=self.new_name)
         return super().leave_Name(original_node, updated_node)
 
+    def is_wrapped_in_call(self, node: CSTNode) -> bool:
+        """Check whether given node is wrapped in Call."""
+        parent = self.resolve_parent_node(node)
+        return m.matches(parent, m.Call())
+
+    def matches_import_scope(self, node: CSTNode) -> bool:
+        """Check whether given node matches the scope of the import."""
+        try:
+            scope = self.resolve_scope(node)
+        except KeyError:
+            # Can't resolve scope of node -> consider no match
+            # Might be because of one of these reasons:
+            # - It's the same name in another scope
+            # - It's a attribute with the same name
+            # - It's a keyword argument
+            return False
+        return scope == self.import_scope
+
 
 class BaseModuleRenameTransformer(BaseRenameTransformer, ABC):
     """Base class to help rename or move a module."""
 
-    @property
-    def old_name(self):
-        return ""
-
-    @property
-    def old_module_parts(self):
-        return self.rename_from.split(".")
-
-    @property
-    def new_name(self):
-        return ""
-
-    @property
-    def new_module_parts(self):
-        return self.rename_to.split(".")
+    def __init__(self, context: CodemodContext) -> None:
+        super().__init__(context)
+        self.old_name = ""
+        self.old_module_parts = self.rename_from.split(".")
+        self.new_name = ""
+        self.new_module_parts = self.rename_to.split(".")
 
 
 class BaseFuncRenameTransformer(BaseRenameTransformer, ABC):
@@ -220,7 +192,7 @@ class BaseFuncRenameTransformer(BaseRenameTransformer, ABC):
 
     def update_call(self, updated_node: Call) -> BaseExpression:
         updated_args = self.update_call_args(updated_node)
-        return Call(args=updated_args, func=Name(self.new_name))
+        return updated_node.with_changes(args=updated_args, func=Name(self.new_name))
 
     def update_call_args(self, node: Call) -> Sequence[Arg]:
         return node.args
